package org.artifactory.metrics.providers.features;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.artifact.lifecycle.ArtifactLifecycleAddon;
import org.artifactory.addon.artifact.lifecycle.ArtifactLifecycleStats;
import org.artifactory.api.callhome.FeatureGroup;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Mor Merhav
 */
@Component
public class ArtifactLifecycleFeature implements CallHomeFeature {

    private static final String MAIN_FEATURE_NAME = "Artifactory On Ice";
    private static final String IS_COLD_INSTANCE = "Is_Cold_Instance";
    private static final String TOTAL_POLICIES_NO = "Total_Policies";
    private static final String LAST_DAY_ARCHIVE_POLICIES_RUNS = "Last_Day_Archive_Policies_Runs";
    private static final String LAST_DAY_ARCHIVED_ARTIFACTS = "Last_Day_Archived_Artifacts";
    private static final String LAST_DAY_ARCHIVED_ARTIFACTS_SIZE = "Last_Day_Archived_Artifacts_Size";
    private static final String LAST_DAY_ARCHIVED_ARTIFACTS_FAILED = "Last_Day_Archived_Artifacts_Failed";
    private static final String LAST_DAY_ARCHIVED_ARTIFACTS_FAILED_SIZE = "Last_Day_Archived_Artifacts_Failed_Size";
    private static final String LAST_DAY_RESTORE_RUNS = "Last_Day_Restore_Runs";
    private static final String LAST_DAY_RESTORE_ARTIFACTS = "Last_Day_Restored_Artifacts";
    private static final String LAST_DAY_RESTORE_ARTIFACTS_SIZE = "Last_Day_Restored_Artifacts_Size";
    private static final String LAST_DAY_RESTORE_ARTIFACTS_FAILED = "Last_Day_Restored_Artifacts_Failed";
    private static final String LAST_DAY_RESTORE_ARTIFACTS_FAILED_SIZE = "Last_Day_Restored_Artifacts_Failed_Size";

    private final AddonsManager addonsManager;

    @Autowired
    public ArtifactLifecycleFeature(AddonsManager addonsManager) {
        this.addonsManager = addonsManager;
    }

    @Override
    public FeatureGroup getFeature() {
        ArtifactLifecycleAddon artifactLifecycleAddon = addonsManager.addonByType(ArtifactLifecycleAddon.class);
        boolean coldInstance = artifactLifecycleAddon.isColdInstance();

        FeatureGroup feature = new FeatureGroup(MAIN_FEATURE_NAME);
        feature.addFeatureAttribute(IS_COLD_INSTANCE, coldInstance);

        if (coldInstance) {
            return feature;
        }

        ArtifactLifecycleStats artifactLifecycleStats = artifactLifecycleAddon.getArtifactLifecycleStats();

        feature.addFeatureAttribute(TOTAL_POLICIES_NO, artifactLifecycleStats.getPoliciesCount());
        feature.addFeatureAttribute(LAST_DAY_ARCHIVE_POLICIES_RUNS, artifactLifecycleStats.getArchiveRunCount());
        feature.addFeatureAttribute(LAST_DAY_ARCHIVED_ARTIFACTS, artifactLifecycleStats.getArchiveSuccessCount());
        feature.addFeatureAttribute(LAST_DAY_ARCHIVED_ARTIFACTS_SIZE, artifactLifecycleStats.getArchiveSuccessSize());
        feature.addFeatureAttribute(LAST_DAY_ARCHIVED_ARTIFACTS_FAILED,
                artifactLifecycleStats.getArchiveFailureCount());
        feature.addFeatureAttribute(LAST_DAY_ARCHIVED_ARTIFACTS_FAILED_SIZE,
                artifactLifecycleStats.getArchiveFailureSize());
        feature.addFeatureAttribute(LAST_DAY_RESTORE_RUNS, artifactLifecycleStats.getRestoreRunCount());
        feature.addFeatureAttribute(LAST_DAY_RESTORE_ARTIFACTS, artifactLifecycleStats.getRestoreSuccessCount());
        feature.addFeatureAttribute(LAST_DAY_RESTORE_ARTIFACTS_SIZE, artifactLifecycleStats.getRestoreSuccessSize());
        feature.addFeatureAttribute(LAST_DAY_RESTORE_ARTIFACTS_FAILED, artifactLifecycleStats.getRestoreFailureCount());
        feature.addFeatureAttribute(LAST_DAY_RESTORE_ARTIFACTS_FAILED_SIZE,
                artifactLifecycleStats.getRestoreFailureSize());

        return feature;
    }
}
