package org.artifactory.repo.system;

import lombok.AllArgsConstructor;
import lombok.Getter;
import org.artifactory.repo.service.RepositoryServiceImpl;

import java.util.Arrays;

/**
 * This class and enum represent the system repos
 * Most system repositories also have their own interceptors in order to block or allow copy/move/delete actions
 * Or any other actions we would like to intercept regarding a specific repo
 * For example {@link org.artifactory.repo.interceptor.SupportBundleRepoInterceptor}
 * Furthermore there are also rebuild functions that the repositories sometimes have here
 * {@link RepositoryServiceImpl} in repositories cache.
 * For UI, there is a RootFetcherStrategy to decide which repos to show.
 * For system repo there is a fetch service and logic regarding the fetch strategy can be added there.
 * {@link org.artifactory.ui.rest.model.artifacts.browse.treebrowser.nodes.fetch.SystemRepoFetchService}
 *
 * @author Tomer Slutsky
 */
public class SystemRepos {
    public static final String CONSUMPTION_REPO_KEY = "jfrog-consumption-logs";
    public static final String LOG_SHIPPING_REPO_KEY = "jfrog-platform-logs";
    public static final String BUILDINFO_REPO_KEY = "artifactory-build-info";
    public static final String PIPEINFO_REPO_KEY = "artifactory-pipe-info";
    public static final String TRASH_REPO_KEY = "auto-trashcan";
    public static final String SUPPORT_BUNDLE_REPO_KEY = "jfrog-support-bundle";
    public static final String IN_TRANSIT_REPO_KEY = "_intransit";
    public static final String EDGE_UPLOADS_REPO_KEY = "artifactory-edge-uploads";
    public static final String RELEASE_BUNDLE_REPO_KEY = "release-bundles";

    @AllArgsConstructor
    @Getter
    public enum SystemRepoConfig {
        BUILDINFO(BUILDINFO_REPO_KEY),
        EDGE_UPLOADS(EDGE_UPLOADS_REPO_KEY),
        IN_TRANSIT(IN_TRANSIT_REPO_KEY),
        PIPEINFO(PIPEINFO_REPO_KEY),
        RELEASE_BUNDLE(RELEASE_BUNDLE_REPO_KEY),
        SUPPORT_BUNDLE(SUPPORT_BUNDLE_REPO_KEY),
        LOG_SHIPPING(LOG_SHIPPING_REPO_KEY),
        CONSUMPTION(CONSUMPTION_REPO_KEY),
        TRASHCAN(TRASH_REPO_KEY);

        private final String repoKey;
        private final boolean visible;

        SystemRepoConfig(String repoKey) {
            this.repoKey = repoKey;
            this.visible = true;
        }

        public static SystemRepoConfig getSystemRepo(String repoKey) {
            return Arrays.stream(SystemRepoConfig.values()).filter(curr -> curr.getRepoKey().equalsIgnoreCase(repoKey))
                    .findFirst()
                    .orElseThrow(() -> new IllegalArgumentException(
                            "Could not find the requested system repo key " + repoKey));
        }
    }
}
