package org.artifactory.addon.artifact.lifecycle;

import org.artifactory.addon.Addon;
import org.artifactory.addon.license.EdgeBlockedAddon;
import org.artifactory.api.rest.artifact.lifecycle.ArtifactLifecycleColdRestoreRequest;
import org.artifactory.api.rest.artifact.lifecycle.ArtifactLifecycleRestoreRequest;
import org.artifactory.aql.result.AqlRestResult;
import org.artifactory.descriptor.artifact.lifecycle.ArtifactLifecyclePolicyDescriptor;
import org.artifactory.rest.exception.MissingRestAddonException;
import org.artifactory.storage.StorageSummary;

import java.io.IOException;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * @author Tamar Rosen
 */
@EdgeBlockedAddon
public interface ArtifactLifecycleAddon extends Addon {

    default void triggerPolicy(String policyKey) {
        throw new MissingRestAddonException();
    }

    default ArtifactLifecyclePolicyModel getPolicy(String policyKey) {
        throw new MissingRestAddonException();
    }

    default ArtifactLifecyclePolicyModel addPolicy(String policyKey, ArtifactLifecyclePolicyDescriptor policy) {
        throw new MissingRestAddonException();
    }

    default ArtifactLifecyclePolicyModel updatePolicy(String policyKey,
            ArtifactLifecyclePolicyDescriptor newPolicy) {
        throw new MissingRestAddonException();
    }

    default void deletePolicy(String policyKey) {
        throw new MissingRestAddonException();
    }

    default void restore(ArtifactLifecycleRestoreRequest restoreRequest) throws IOException {
        throw new MissingRestAddonException();
    }

    default void executeColdRestore(ArtifactLifecycleColdRestoreRequest coldRestoreRequest) {
        throw new MissingRestAddonException();
    }

    default Map<String, String> getCreateBinaryHeaders() {
        return Collections.emptyMap();
    }

    default Map<String, String> getInitiateBinaryRestoreHeaders() {
        return Collections.emptyMap();
    }

    default Map<String, String> getIsBinaryRestoredHeaders() {
        return Collections.emptyMap();
    }

    default List<ArtifactLifecyclePolicyModel> getPolicies() {
        throw new MissingRestAddonException();
    }

    default ArtifactLifecycleStats getArtifactLifecycleStats() {
        throw new MissingRestAddonException();
    }

    default boolean isColdInstance() {
        return false;
    }

    default AqlRestResult searchOnColdInstance(String aqlStr) {
        throw new MissingRestAddonException();
    }

    default String generateJFrogColdStorageNamespace() {
        return null;
    }

    default void coldStorageHandshake() {
        throw new MissingRestAddonException();
    }

    default String getJfrogColdStorageArchivingNamespace() {
        return null;
    }

    default List<ArtifactLifecycleArchivePolicyStatus> getArchivePoliciesExecutions(Long extractLongEpoch,
            Long extractLongEpoch1, String searchStr) {
        throw new MissingRestAddonException();
    }

    default List<ArtifactLifecycleRestorePolicyStatus> getRestorePoliciesExecutions(Long extractLongEpoch,
            Long extractLongEpoch1) {
        throw new MissingRestAddonException();
    }

    default StorageSummary getArchiveStorageSummary() {
        throw new MissingRestAddonException();
    }

    default ArtifactLifecycleLinkModel getArchiveStorageLink() {
        throw new MissingRestAddonException();
    }

    default boolean isRetentionEnabled() {
        return false;
    }
}