#!/usr/bin/env bash

set -e

export PROJECT_ROOT_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" && cd ../../.. && pwd)"
source "${PROJECT_ROOT_DIR}/build/ci/scripts/lib/commonUtils.sh"
source "${PROJECT_ROOT_DIR}/build/ci/scripts/lib/mavenUtils.sh"
source "${PROJECT_ROOT_DIR}/build/ci/scripts/lib/dockerUtils.sh"
source "${PROJECT_ROOT_DIR}/build/ci/scripts/lib/versionUtils.sh"
source "${PROJECT_ROOT_DIR}/build/ci/scripts/lib/pipelineUtils.sh"
source "${PROJECT_ROOT_DIR}/build/ci/scripts/lib/buildInfoUtils.sh"
source "${PROJECT_ROOT_DIR}/build/ci/scripts/lib/rpmUtils.sh"


function on_start() {
  cd "${PROJECT_ROOT_DIR}/artifactory-oss"
  setCustomBuildTypes
  buildInfo_setBuildNameAndNumber
  ${PROJECT_ROOT_DIR}/build/ci/scripts/javaGenerateConfig.sh

  export MVN_COMMAND="install -P${ARTIFACTORY_OSS_BUILD_TYPES} -P'!generate-test-package' -DskipTests ${MAVEN_CONFIG}"

  cd "${PROJECT_ROOT_DIR}"
  local version targetVersion revision
  version="$(maven_getVersionFromPom "artifactory-oss/pom.xml")"
  echo "Version: ${version}"

  if version_isRelease "${version}"; then
    revision=$(version_getRevisionFromVersion ${version})
    export MVN_COMMAND="${MVN_COMMAND} -Dartifactory.revision.prop=${revision}"
  fi
}

function on_execute() {
  return 0
}

function on_complete() {
  return 0
}

function on_success() {
  # Publish the build with the Maven artifacts
  local version="$(maven_getVersionFromPom "${PROJECT_ROOT_DIR}/artifactory-oss/pom.xml")"

  buildInfo_modifyBuildName "${JFROG_CLI_BUILD_NAME}" "${JFROG_CLI_BUILD_NUMBER}"
  buildInfo_publish "${JFROG_CLI_BUILD_NAME}"
  write_output artifactory_oss_maven_package_build_info buildName="${JFROG_CLI_BUILD_NAME}" buildNumber="${JFROG_CLI_BUILD_NUMBER}" version="${version}"
}

# exports an environment variable with desired build types based on the last git commit body or env variable
function setCustomBuildTypes(){
  local branch
  branch="$(pipeline_gitBranchName)"
  add_run_variables ARTIFACTORY_OSS_BUILD_TYPES="release"
  export_run_variables
}