package org.artifactory.storage.db.aql.itest.service.decorator;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.artifact.lifecycle.ArtifactLifecycleAddon;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.aql.AqlException;
import org.artifactory.aql.model.AqlDomainEnum;
import org.artifactory.aql.model.AqlOverloadSettings;
import org.artifactory.aql.model.AqlPhysicalFieldEnum;
import org.artifactory.aql.model.AqlValue;
import org.artifactory.aql.result.rows.AqlRowResult;
import org.artifactory.storage.db.aql.sql.builder.query.aql.AqlQuery;
import org.artifactory.storage.db.aql.sql.builder.query.aql.AqlQueryElement;
import org.artifactory.storage.db.aql.sql.builder.query.aql.Criterion;

import java.util.List;
import java.util.Optional;

/**
 * @author Omri Ziv
 */
public class ArtifactLifecycleColdSearchDecorator implements DecorationStrategy {

    @Override
    public <T extends AqlRowResult> void decorate(AqlQuery<T> aqlQuery, AqlQueryDecoratorContext decoratorContext) {
        if (!isColdSearch(decoratorContext)) {
            return;
        }
        if (AqlDomainEnum.items != aqlQuery.getDomain()) {
            throw new AqlException("Artifact Lifecycle Aql search is not supported non items search");
        }
        List<AqlQueryElement> queryElements = aqlQuery.getElementsForNative();
        int index = findNextFieldCriteria(0, AqlPhysicalFieldEnum.itemRepo, queryElements);
        while (index > -1) {
            Criterion currCriterion = (Criterion) queryElements.get(index);
            AqlValue variable2 = (AqlValue) currCriterion.getVariable2();
            String repoKey = variable2.getValue();
            variable2.setValue(addNamespacePrefix(repoKey));
            index = findNextFieldCriteria(index + 1, AqlPhysicalFieldEnum.itemRepo, queryElements);
        }
    }

    private static boolean isColdSearch(AqlQueryDecoratorContext decoratorContext) {
        return Optional.ofNullable(decoratorContext)
                .map(AqlQueryDecoratorContext::getAqlOverloadSettings)
                .map(AqlOverloadSettings::getStreamFunc)
                .isPresent();
    }

    private static String addNamespacePrefix(String repo) {
        AddonsManager addonsManager = ContextHelper.get().beanForType(AddonsManager.class);
        ArtifactLifecycleAddon artifactLifecycleAddon = addonsManager.addonByType(ArtifactLifecycleAddon.class);
        return artifactLifecycleAddon.getJfrogColdStorageArchivingNamespace() + "_" + repo;
    }

}
