package org.artifactory.storage.db.conversion.version.v227;

import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.artifactory.storage.db.conversion.DbConversionException;
import org.artifactory.storage.db.version.ConversionPredicate;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JdbcHelper;

import java.util.function.BiPredicate;

import static org.jfrog.storage.util.DbUtils.indexExists;

/**
 * @author Volodymyr Shulga
 */
@Slf4j
public class V227NodeEventsErrorDropIndexConversionPredicate implements ConversionPredicate {

    @NonNull
    @Override
    public BiPredicate<JdbcHelper, DbType> condition() {
        return (jdbcHelper, dbType) -> {
            try {
                if (!indexExists(jdbcHelper, "node_events_errors", "operation_key", "node_events_errors_op_key_idx",
                        dbType, true)) {
                    log.info("Index 'node_events_errors_op_key_idx' doesn't exist for table 'node_events_errors' " +
                            "- Skipping upgrade.");
                    return false;
                }
            } catch (Exception e) {
                log.error("Cannot run conversion 'v227' - Failed to retrieve schema metadata");
                throw new DbConversionException(
                        "Cannot run conversion 'v227' - Failed to retrieve schema metadata: " + e.getMessage(), e);
            }
            log.info("Dropping index 'node_events_errors_op_key_idx' on table 'node_events_errors'");
            return true;
        };
    }
}
