package org.artifactory.ui.rest.service.admin.configuration.layouts;

import org.artifactory.api.repo.RepositoryService;
import org.artifactory.descriptor.repo.FederatedRepoDescriptor;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.descriptor.repo.RepoLayout;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.ui.rest.model.admin.configuration.layouts.RepositoryLayoutModel;
import org.testng.annotations.Test;

import java.util.ArrayList;
import java.util.List;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;

/**
 * @author Alexei Vainshtein
 */
public class GetLayoutInfoServiceTest {

    @Test
    public void testCreateAssociations() {
        RepoLayout defaultLayout = createRepoLayout("default");
        RepositoryService repositoryService = mock(RepositoryService.class);
        createLocalRepositories(defaultLayout, repositoryService);
        createFederatedRepositories(defaultLayout, repositoryService);

        GetLayoutInfoService service = new GetLayoutInfoService(null, repositoryService);
        RepositoryLayoutModel layoutModel = new RepositoryLayoutModel(defaultLayout);
        service.createAssociations(layoutModel);
        assertEquals(layoutModel.getRepositoryAssociations().getLocalRepositories().size(), 4);
    }

    private void createFederatedRepositories(RepoLayout defaultLayout, RepositoryService repositoryService) {
        List<FederatedRepoDescriptor> federatedRepoDescriptors = new ArrayList<>();
        federatedRepoDescriptors.add(createFederatedRepo("test4", RepoType.Generic, defaultLayout));
        federatedRepoDescriptors.add(createFederatedRepo("test5", RepoType.Npm, defaultLayout));
        federatedRepoDescriptors.add(createFederatedRepo("test6", RepoType.Docker, createRepoLayout("non-def")));
        when(repositoryService.getFederatedRepoDescriptors()).thenReturn(federatedRepoDescriptors);
    }

    private void createLocalRepositories(RepoLayout defaultLayout, RepositoryService repositoryService) {
        List<LocalRepoDescriptor> localRepoDescriptors = new ArrayList<>();
        localRepoDescriptors.add(createLocalRepo("test", RepoType.Generic, defaultLayout));
        localRepoDescriptors.add(createLocalRepo("test2", RepoType.Docker, defaultLayout));
        localRepoDescriptors.add(createLocalRepo("test3", RepoType.Npm, createRepoLayout("non-def")));
        when(repositoryService.getLocalRepoDescriptors()).thenReturn(localRepoDescriptors);
    }

    private LocalRepoDescriptor createLocalRepo(String key, RepoType type, RepoLayout repoLayout) {
        LocalRepoDescriptor localRepoDescriptor = new LocalRepoDescriptor();
        localRepoDescriptor.setKey(key);
        localRepoDescriptor.setType(type);
        localRepoDescriptor.setRepoLayout(repoLayout);
        return localRepoDescriptor;
    }

    private FederatedRepoDescriptor createFederatedRepo(String key, RepoType type, RepoLayout repoLayout) {
        FederatedRepoDescriptor federatedRepoDescriptor = new FederatedRepoDescriptor();
        federatedRepoDescriptor.setKey(key);
        federatedRepoDescriptor.setType(type);
        federatedRepoDescriptor.setRepoLayout(repoLayout);
        return federatedRepoDescriptor;
    }

    private RepoLayout createRepoLayout(String name) {
        RepoLayout repoLayout = new RepoLayout();
        repoLayout.setName(name);
        return repoLayout;
    }
}