package org.artifactory.ui.rest.service.builds.search;

import org.apache.commons.lang3.StringUtils;
import org.artifactory.addon.build.BuildAddon;
import org.artifactory.api.build.BuildService;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.projects.ProjectsService;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

import java.util.Arrays;
import java.util.Collection;

import static org.artifactory.repo.system.SystemRepos.BUILDINFO_REPO_KEY;
import static org.mockito.Mockito.when;

@RunWith(Parameterized.class)
public class SearchBuildIDsByNameServiceTest {

    private final static String PROJECT = "proj";
    private final static String PROJECT_BUILD_REPO = "proj" + BuildAddon.BUILD_INFO_REPO_EXTENSION;

    private SearchBuildIDsByNameService searchBuildIDsByNameService;
    @Mock
    private BuildService buildService;
    @Mock
    private AuthorizationService authService;

    private String buildRepo;
    private String projectKey;
    private String expectedBuildRepo;

    public SearchBuildIDsByNameServiceTest(String buildRepo, String projectKey, String expectedBuildRepo) {
        this.buildRepo = buildRepo;
        this.projectKey = projectKey;
        this.expectedBuildRepo = expectedBuildRepo;
    }

    @Before
    public void setUp() {
        MockitoAnnotations.openMocks(this);
        searchBuildIDsByNameService = Mockito.spy(new SearchBuildIDsByNameService());
        searchBuildIDsByNameService.setBuildService(buildService);
        searchBuildIDsByNameService.setAuthService(authService);
        when(buildService.normaliseBuildRepoName(StringUtils.EMPTY)).thenReturn(BUILDINFO_REPO_KEY);
        when(buildService.normaliseBuildRepoName(PROJECT_BUILD_REPO)).thenReturn(PROJECT_BUILD_REPO);
        when(buildService.normaliseBuildRepoName(BUILDINFO_REPO_KEY))
                .thenReturn(BUILDINFO_REPO_KEY);
        when(buildService.getBuildInfoRepoForDefaultProject()).thenReturn(BUILDINFO_REPO_KEY);
        when(buildService.getBuildInfoRepoKeyByProject(PROJECT)).thenReturn(PROJECT_BUILD_REPO);
        when(buildService.getBuildInfoRepoKeyByProject(ProjectsService.DEFAULT_PROJECT))
                .thenReturn(BUILDINFO_REPO_KEY);
    }

    @Test
    public void testGetBuildRepoForSearch() {
        String resultRepo = searchBuildIDsByNameService.getBuildRepoForSearch(buildRepo, projectKey);
        Assert.assertEquals(expectedBuildRepo, resultRepo);
    }


    @Parameterized.Parameters(name = "Provided buildRepo and projectKey: {0}, {1}")
    public static Collection provideQueryParams() {
        return Arrays.asList(new Object[][] {
                {StringUtils.EMPTY, StringUtils.EMPTY, null},
                {BUILDINFO_REPO_KEY, StringUtils.EMPTY, BUILDINFO_REPO_KEY},
                {StringUtils.EMPTY, ProjectsService.DEFAULT_PROJECT, BUILDINFO_REPO_KEY},
                {StringUtils.EMPTY, PROJECT, PROJECT_BUILD_REPO},
                {StringUtils.EMPTY, ProjectsService.UNASSIGNED, BUILDINFO_REPO_KEY},
                {PROJECT_BUILD_REPO, PROJECT, PROJECT_BUILD_REPO},
                {PROJECT_BUILD_REPO, ProjectsService.DEFAULT_PROJECT, PROJECT_BUILD_REPO}
        });
    }

}