package org.artifactory.rest.resource.artifact.lifecycle;

import org.artifactory.addon.artifact.lifecycle.ArtifactLifecyclePolicyBaseModel;
import org.artifactory.api.rest.artifact.lifecycle.ArtifactLifecycleRestoreRequest;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.rest.common.resource.BaseResource;
import org.artifactory.rest.services.artifact.lifecycle.ArtifactLifecycleArchiveServiceFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.annotation.security.RolesAllowed;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import java.util.List;

import static javax.ws.rs.core.MediaType.APPLICATION_JSON;

/**
 * @author Omri Ziv
 */
@Path("artifactLifecycleArchive")
@Component
@RolesAllowed({AuthorizationService.ROLE_ADMIN})
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class ArtifactLifecycleUIResource extends BaseResource {

    @Autowired
    private ArtifactLifecycleArchiveServiceFactory artifactLifecycleArchiveServiceFactory;

    @GET
    @Path("policies")
    @Produces(APPLICATION_JSON)
    public Response getAllPolicies() {
        return runService(artifactLifecycleArchiveServiceFactory.getAllArtifactLifecyclePolicyService());
    }

    @GET
    @Path("policies/{policyKey}")
    @Produces(APPLICATION_JSON)
    public Response getPolicy() {
        return runService(artifactLifecycleArchiveServiceFactory.getArtifactLifecyclePolicyService());
    }

    @POST
    @Path("policies")
    @Consumes(APPLICATION_JSON)
    public Response createPolicy(ArtifactLifecyclePolicyBaseModel policy) {
        return runService(artifactLifecycleArchiveServiceFactory.createArtifactLifecyclePolicyService(), policy);
    }

    @DELETE
    @Path("policies/{policyKey}")
    public Response deletePolicy() {
        return runService(artifactLifecycleArchiveServiceFactory.deleteArtifactLifecyclePolicyService());
    }

    @DELETE
    @Path("policies")
    @Consumes(APPLICATION_JSON)
    public Response deletePolicies(List<ArtifactLifecyclePolicyBaseModel> policy) {
        return runService(artifactLifecycleArchiveServiceFactory.deleteArtifactLifecyclePolicyService());
    }

    @PUT
    @Path("policies")
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.APPLICATION_JSON)
    public Response updatePolicy(ArtifactLifecyclePolicyBaseModel policy) {
        return runService(artifactLifecycleArchiveServiceFactory.updateArtifactLifecyclePolicyService(), policy);
    }

    @POST
    @Path("restore")
    @Consumes(APPLICATION_JSON)
    public Response restore(ArtifactLifecycleRestoreRequest restoreRequest) {
        return runService(artifactLifecycleArchiveServiceFactory.artifactLifecycleRestoreService(), restoreRequest);
    }

}
